#include "../rk4fixed.h"
#include <iostream>
#include <sys/time.h>

using std::cout;
using std::cin;
using std::endl;


typedef double FLOAT_TYPE; //you can try 'long double' and 'float' instead.


namespace job_1 { //First job for solving ======================================

//system for solving
void s(FLOAT_TYPE t, FLOAT_TYPE x[], FLOAT_TYPE rp[]) {
    rp[0] = 2.0*t;    //means dx0/dt = 2t
    rp[1] = 3.0*x[0]; //means dx1/dt = 3x0
}

const int num = 2;  //count of eq. in sys.
FLOAT_TYPE iC[num] = {0.0, 0.0}; //init. cond. for system
FLOAT_TYPE t0 = 0.0, te = 10.0; //beg. and end of calc. range

}//=============================================================================


namespace job_2 { //Second job for solving =====================================

//system for solving
void s (FLOAT_TYPE t, FLOAT_TYPE x[], FLOAT_TYPE rp[]) {
    rp[0] = 2.0*t;
    //means dx0/dt = 2t
    rp[1] = 3.0*(x[0]+t*t)/2.0;
    //means dx1/dt = 3(x0 + t^2)/2
    rp[2] = 4.0*(x[1]+t*x[0]+t*t*t)/3.0;
    //means dx2/dt =4(x1 + t*x0 + t^3)/3
    rp[3] = 5.0*(x[2]+t*x[1]+t*t*x[0]+t*t*t*t)/4.0;
    //means dx3/dt = 5(x2 + t*x1 + t^2*x0 + t^4)/4
}

const int num = 4;  //count of eq. in sys.
FLOAT_TYPE iC[num] = {0.0, 0.0, 0.0, 0.0}; //init. cond. for system
FLOAT_TYPE t0 = 0.0, te = 10.0; //beg. and end of calc. range

}//=============================================================================


//Help functions
unsigned long GetTickCount(); //crosplatf. analog of Win32Api
void print_table(FLOAT_TYPE* Ar, FLOAT_TYPE *Ar2D[], int cnt, int pts); //prints results


int main() {

    int i, st, points, k;
    int erc;
    unsigned long dt;
    FLOAT_TYPE *tOut;
    FLOAT_TYPE *XOuts1[job_1::num], *XOuts2[job_2::num];


    for(;;) {

        cout << "Enter number of steps for output [-1 for exit]\n";
        cin >> st;
        if (st == -1) break;

        points = st + 1;

        tOut = new FLOAT_TYPE [points];
        for(i = 0; i < job_1::num; ++i)
            XOuts1[i] = new FLOAT_TYPE [points];
        for(i = 0; i < job_2::num; ++i)
            XOuts2[i] = new FLOAT_TYPE [points];


        cout << "Enter factor for steps for calc.\n";
        cin >> k;

        {//Solving of first problem:
            using namespace job_1;
            dt = GetTickCount();
            erc = rk4fixed<FLOAT_TYPE,num>(s, iC, t0, te, tOut, XOuts1, points, k);
            //result must be: x0 = t^2, x1 = t^3
            dt = GetTickCount() - dt;
            if (!erc) {
                cout << "\nTime, ms = " << dt << endl;
                cout << "Returned with no errors\n\n";
                print_table(tOut, XOuts1, num, points);
            } else cout << "\nReturned code is " << erc << endl;
        }

        {//Solving of 2-nd problem:
            using namespace job_2;
            dt = GetTickCount();
            erc = rk4fixed<FLOAT_TYPE,num>(s, iC, t0, te, tOut, XOuts2, points, k);
            //result must be: x0 = t^2, x1 = t^3, x2 = t^4, x3 = t^5
            dt = GetTickCount() - dt;
            if (!erc) {
                cout << "\nTime, ms = " << dt << endl;
                cout << "Returned with no errors\n\n";
                print_table(tOut, XOuts2, num, points);
            } else cout << "\nReturned code is " << erc << endl;
        }

        //clean memory:
        delete [] tOut;
        for(i = 0; i < job_1::num; ++i) delete[] XOuts1[i];
        for(i = 0; i < job_2::num; ++i) delete[] XOuts2[i];

        cout << endl;

    } //end for(;;)

    return 0;
}


unsigned long GetTickCount() {
    timeval tv;
    gettimeofday(&tv, NULL);
    return (tv.tv_sec*1000 + tv.tv_usec/1000);
}

void print_table(FLOAT_TYPE* Ar, FLOAT_TYPE *Ar2D[], int cnt, int pts) {
    int i,j;
    cout << "t = ";
    for (i = 0; i < cnt; i++)
        cout << '\t' << "x" << i << " = ";
    cout << endl;
    for(i = 0; i < pts; ++i) {
        cout << Ar[i];
        for (j = 0; j < cnt; j++)
            cout << '\t' << Ar2D[j][i];
        cout << endl;
    }
}
