

/******************************************************************************\
*                                                                              *
*                       rk4fixed-cpp v.2.00beta                                *
*                                                                              *
*    The C++ module for solving ordinary dif. equations and their systems      *
*                         by Runge-Kutta Method                                *
*                                                                              *
*    Beta (testing) version on nov. 2013                                       *
*    'As-Is', OpenSource                                                       *
*                                                                              *
*    Andrey G. Sadovoy                                                         *
*    sadovoya@mtu-net.ru                                                       *
*    sadovoya.narod.ru                                                         *
*    sintreseng.narod.ru                                                       *
*                                                                              *
\******************************************************************************/


#ifndef RK4FIXED_H_INCLUDED
#define RK4FIXED_H_INCLUDED

template < typename T, int count > //count is a count of equations (template param.)
int rk4fixed(void (*syst)(T t, T* x, T* rightParts), //system
             T* initConds,    //initial conditions (array)
             T first, T last, //initial and last points
             T* tOutPoints, T** xOutMatrix, //results (arrays 1 & 2 dim.)
             int pointsNumb,      //number of points for output
             int stepsFact = 1) { //factor for steps inside of method

    if (  (!count) || (pointsNumb <= 1) || (!stepsFact)  ) return 1000;
    // Error Code 1000: Bad data.

    if (last < first) return 2000;
    // Error Code 2000: Last point Must be not less Fist point.

    const int steps = (pointsNumb - 1)*stepsFact;
    const T delt = (last - first)/T(steps); // length of step
    const T halfDelt = 0.5*delt;
    const T one_6_delt = delt/6.0;

    //static arrays due to count is template param.
    T varX[count], tmpX[count];
    T c1[count], c2[count], c3[count], c4[count];

    int i, j, i_out;

    // Initial values of vars and first point of results:
    T t = tOutPoints[0] = first;
    for (j = 0; j < count; ++j)
        xOutMatrix[j][0] = varX[j] = initConds[j];

    for (i_out = 1; i_out < pointsNumb; ++i_out) {
        for (i = 0; i < stepsFact; ++i) {
            syst(t, varX, c1);
            for (j = 0; j < count; ++j)
                tmpX[j] = varX[j] + c1[j]*halfDelt;
            t += halfDelt;
            syst(t, tmpX, c2);
            for (j = 0; j < count; ++j)
                tmpX[j] = varX[j] + c2[j]*halfDelt;
            syst(t, tmpX, c3);
            for (j = 0; j < count; ++j)
                tmpX[j] = varX[j] + c3[j]*delt;
            t += halfDelt;
            syst(t, tmpX, c4);
            for (j = 0; j < count; ++j)
                varX[j] += (c1[j]+(c2[j]+c3[j])+(c2[j]+c3[j])+c4[j])*one_6_delt;
        }
        tOutPoints[i_out] = t;
        for (j = 0; j < count; ++j)
            xOutMatrix[j][i_out] = varX[j];
    }

    return 0; // Error Code 0: No errors.
}

#endif // RK4FIXED_H_INCLUDED
